# An UDP client that can send data

import socket
import thread
from time import sleep

class UDPClient:
  def __init__(self, ip, port):
    self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    self.sendFlag = 0
    self.stopSendThread = 0
    self.sendBuffer = []
    self.sendLock = thread.allocate_lock()
    self.granularity = 1.0 / 60.0

  # Start the sending thread
  def start(self):
    self.stopSendThread = 0    
    thread.start_new_thread(self.__send, ())
    
  # Stop the sending thread
  def stop(self):
    self.stopSendThread = 1
    
  # Place some data in the sending queue
  def send(self, data, addr):
    self.sendLock.acquire()
    self.sendBuffer.append((data, addr))
    self.sendFlag = 1
    self.sendLock.release()

  ### Private methods ###
  
  # The sending loop
  def __send(self):
    while not self.stopSendThread:
      if self.sendFlag:
        self.sendLock.acquire()
        sendBuffer = self.sendBuffer
        self.sendBuffer = []
        self.sendFlag = 0
        self.sendLock.release()
        for data, addr in sendBuffer:
          self.socket.sendto(data, addr)
      sleep(self.granularity)
